@@ -439,26 +439,14 @@ RESULTS_EXTEND_HELP = '''\
     database. This has no effect if the results contain only 1-grams.'''
 RESULTS_EPILOG = '''\
     If more than one modifier is specified, they are applied in the
-    following order: --extend, --bifurcated-extend,
-    --denormalise-corpus, --denormalise_mapping, --reduce,
-    --reciprocal, --excise, --zero-fill, --ngrams, --min/max-works,
-    --min/max-size, --min/max-count, --min/max-count-work, --remove,
-    --relabel, --sort. All of the options that modify the format are
-    performed at the end, and only one should be specified. The one
-    exception to this is denormalisation, which adds a column to the
-    results without disrupting any other operations.
-
-    It is important to be careful with the use of --reduce. Coupled
-    with filters such as --max-size, --min-count, etc, many results
-    may be discarded without trace (since the reduce occurs
-    first). Note too that performing "reduce" on a set of results more
-    than once will make the results inaccurate! Denormalisation should
-    always be done before reducing results.
-
-    The denormalisation options together produce a set of results with
-    all denormalised forms that occur in each witness presented, along
-    with an extra column, "{}", giving the normalised form each was
-    derived from.
+    following order: --extend, --bifurcated-extend, --denormalise,
+    --reduce, --reciprocal, --excise, --zero-fill, --ngrams,
+    --min/max-works, --min/max-size, --min/max-count,
+    --min/max-count-work, --remove, --relabel, --sort. All of the
+    options that modify the format are performed at the end, and only
+    one should be specified. The one exception to this is
+    denormalisation, which adds a column to the results without
+    disrupting any other operations - but see below.
 
     --extend applies before --reduce because it may generate results
     that are also amenable to reduction.
@@ -467,6 +455,21 @@ RESULTS_EPILOG = '''\
     at least two labels in the results in order to give correct
     results.
 
+    The denormalisation options together produce a set of results with
+    all denormalised forms that occur in each witness presented, along
+    with an extra column, "{}", giving the normalised form each was
+    derived from. Since denormalised intersect results may no longer
+    conform to normal intersect rules (that each n-gram occurs at
+    least once within each label), running some further operations
+    (such as extend) is likely to cause unwanted removal of
+    results.
+
+    It is important to be careful with the use of --reduce. Coupled
+    with filters such as --max-size, --min-count, etc, many results
+    may be discarded without trace (since the reduce occurs
+    first). Note too that performing "reduce" on a set of results more
+    than once will make the results inaccurate!
+
     --min-count and --max-count set the range within which the total
     count of each n-gram, across all works, must fall. For each work,
     its count is taken as the highest count among its witnesses.
@@ -549,7 +552,8 @@ RESULTS_SORT_HELP = 'Sort the results.'
 RESULTS_UNSAFE_GROUP_TITLE = 'format changing arguments'
 RESULTS_UNSAFE_GROUP_DESCRIPTION = '''\
     These arguments change the format of the results, making them
-    potentially unsafe to use other operations on.'''
+    potentially unsafe to use other operations on, or causing such
+    operations to fail.'''
 RESULTS_ZERO_FILL_HELP = '''\
     Add rows with a count of 0 for each n-gram in each witness of a
     work that has at least one witness bearing that n-gram.'''
@@ -710,7 +714,7 @@ EXCISE_OVERWRITE_WORK_WARNING = ('Output work directory "{}" already exists;'
 INSUFFICIENT_LABELS_QUERY_ERROR = (
     'Not running query with fewer than two defined labels.')
 LABEL_NOT_IN_CATALOGUE_ERROR = (
-    'Supplied label is not present in the supplied catalogue.')
+    'Supplied label "{}" is not present in the supplied catalogue.')
 MISSING_DATA_STORE_ERROR = (
     'Data store does not exist or is inaccessible at {}.')
 MISSING_REQUIRED_COLUMNS_ERROR = (
